#!/bin/env lua
--

local function err(s)
  print("ERROR: "..(s or "Unknown error."))
  os.exit(1)
end

local fni = (arg and arg[1]) or err("No input file.")

local fi = io.open(fni,"rb")
if not fi then
  err("Can't open: "..fni)
end

local header = string.char(22,22,22)
local data = fi:read("*all")
fi:close()

local function printbx(x)
  io.write(string.format("%.2X ", x))
end

local function printwx(x)
  io.write(string.format("%.4X ", x))
end

--[[
tape header:
db  $16,$16,$16,$24   ;
db  $00             ; integers flag (true if != 0x00)
db  $00             ; strings flag  (true if != 0x00)
db  $80             ; type          (0x00 for BASIC, 0x40 for arrays, 0x80 for data)
db  $01             ; auto flag     (true if != 0x00)
db  End / 256       ; end address   (MSB)
db  End % 256       ; end address   (LSB)
db  Start / 256       ; start address (MSB)
db  Start % 256       ; start address (LSB)
db  $00             ; unused
db  $00             ; filename
**/
--]]
local part = 0

local function write_bin(b, s, ss, e, name)
  name = (name ~= "" and name) or "NONAME"
  local fo = io.open("["..name.."]-"..tonumber(part).."-"..string.format("%.4X-%.4X",ss,e)..".bin", "wb")
  if(fo) then
    local s = data:sub(b,b+s)
    fo:write(s)
    fo:close()
  end
end

local function parse_header(b)
  local _start = (256 * data:byte(b+6) + data:byte(b+7))
  local _end = (256 * data:byte(b+4) + data:byte(b+5))
  local _size = _end - _start
  local _type = data:byte(b+2)
  io.write("type:"); printbx(_type)
  io.write("start:"); printwx(_start)
  io.write("end:"); printwx(_end)
  io.write("size:", _size)
  --
  local x = 9
  local _name = ""
  io.write("\tname:");
  while 0x00 ~= data:byte(b+x) do
    io.write(string.char(data:byte(b+x)))
    _name = _name .. string.char(data:byte(b+x))
    x = x + 1
  end
  io.write("\t");
  --
  if 0x80 == _type then
    write_bin(b+x+1, _size+5, _start, _end, _name)
  elseif 0x00 == _type then
    write_bin(b+x+1, _size+5, _start, _end, _name)
  elseif 0x40 == _type then
    write_bin(b+x+1, _size+5, _start, _end, _name)
  else
    write_bin(b+x+1, _size+5, _start, _end, _name)
    io.write("WARN: unknown type:")
    printbx(_type)
  end
  part = part + 1
end

local b,e = string.find(data, header)
while b do
  if 22 ~= data:byte(e+1) then
    --
    io.write(tostring(part).." | ")
    printwx(b)
    io.write(".. ")
    printwx(e)
    io.write("| ")
    printbx(data:byte(e+1))
    io.write("| ")
    --
    parse_header(e+2)
    print""
  end
  b,e = string.find(data, header, b+1)
end
